using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using XDOP.schemas_coscom_org.device;
using System.Threading;
using System.IO;

namespace dotNET_Controls_Sample
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            // In case the coscom panel is a direct child of the containing form
            // you have to call this method in order to proper scaling of the controls.
            coscomPanel2.AfterDesignerInit();
        }

        private void buttonConnect_Click(object sender, EventArgs e)
        {
            // Search the device in a new thread
            Thread t = new Thread(new ParameterizedThreadStart(Connect));
            t.Start(new object[] { textBoxPort.Text,int.Parse(comboBoxBaudrate.Text) });

        }

        IDMCU5Treadmill mcu5device = new MCU5Treadmill();
        private void Connect(object args)
        {
            try
            {
                // Set XDOP Cache active
                XDOP.XDOPCore.GetInstance().UseDescriptionCache = true;
                // Set XDOP Cache to another location than default path
                XDOP.XDOPCore.GetInstance().DescriptionCachePath = Path.Combine(Application.StartupPath, "XDOP");
                // Create a new protocol parameter instance
                XDOP.ProtocolParameter ppara = new XDOP.ProtocolParameter(true, true, 4000, 3000, 18000, 0, 0);
                // Create a new serial connection parameter for the specified com port and baud rate
                XDOP.ConnectionParameterCOM comparameter = 
                    new XDOP.ConnectionParameterCOM((string)(((object[])args)[0]), (int)(((object[])args)[1]));
                // Get a transport object by using the connection parameter
                XDOP.ITransportObject itransport = XDOP.TransportObjectCOM.getIXDOPComs(comparameter)[0];

                // connect to the device
                mcu5device.Connect(itransport, ppara);

                // for a save communication you should enable the messgae checksum on device side by invoking the 
                // SetProtocolParams action in the TreadmillGeneralConfig service
                
                // You also should use the failsafe component in your application as done in this small sample
               
                // Inform all coscom controls within the coscom panels that a new device connection was made.
                // The second bool parameter defines that coscom components such as the buzzer and failsafe control
                ///on the form will be set to.
                coscomPanel1.SetDevice(mcu5device, true);
                coscomPanel2.SetDevice(mcu5device, true);

                // Manually set the device for the controls that are not located in a coscom panel
                _DigitDisplaySpeed.setDevice(mcu5device);
                uI_ButtonMinus.setDevice(mcu5device);
                uI_ButtonPlus.setDevice(mcu5device);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error: "+ex.Message);
            }
        }

        private void buttonDisconnect_Click(object sender, EventArgs e)
        {
            // Dispose the device
            if (mcu5device != null)
                mcu5device.DisposeDevice();
            mcu5device = null;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (mcu5device != null)
                mcu5device.DisposeDevice();
        }
    }
}