using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using XDOP.schemas_coscom_org.device;
using System.Threading;
using XDOP.schemas_coscom_org.service;
using System.IO;

namespace dotNET_Objects_Sample
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }


        private void buttonConnect_Click(object sender, EventArgs e)
        {
            // Search the device in a new thread
            Thread t = new Thread(new ParameterizedThreadStart(Connect));
            t.Start(new object[] { textBoxPort.Text, int.Parse(comboBoxBaudrate.Text) });

        }

        IDMCU5Treadmill mcu5device = new MCU5Treadmill();
        XDOP.schemas_coscom_org.service.ISTreadmillDirectControl IServiceDirectcontrol = null;
        private void Connect(object args)
        {
            try
            {
                // Set XDOP Cache active
                XDOP.XDOPCore.GetInstance().UseDescriptionCache = true;
                // Set XDOP Cache to another location than default path
                XDOP.XDOPCore.GetInstance().DescriptionCachePath = Path.Combine(Application.StartupPath, "XDOP");
                // Create a new protocol parameter instance
                XDOP.ProtocolParameter ppara = new XDOP.ProtocolParameter(true, true, 4000, 3000, 18000, 0, 0);
                // Create a new serial connection parameter for the specified com port and baud rate
                XDOP.ConnectionParameterCOM comparameter =
                    new XDOP.ConnectionParameterCOM((string)(((object[])args)[0]), (int)(((object[])args)[1]));
                // Get a transport object by using the connection parameter
                XDOP.ITransportObject itransport = XDOP.TransportObjectCOM.getIXDOPComs(comparameter)[0];

                // connect to the device
                mcu5device.Connect(itransport, ppara);

                // iterate through all services and save a reference to the direct control service
                foreach (CodeGenerator.IServiceObject service in mcu5device.Services)
                {
                    if (service.ServiceType.Equals(TreadmillDirectControl.CodeGeneratorServiceType))
                        IServiceDirectcontrol = (ISTreadmillDirectControl)service;
                }

                // subscribe to changes of the actual speed and grade by adding the following event handlers
                IServiceDirectcontrol.ActualGradeChanged += 
                    new OnChangeTreadmillDirectControlActualGrade(IServiceDirectcontrol_ActualGradeChanged);
                IServiceDirectcontrol.ActualSpeedChanged += 
                    new OnChangeTreadmillDirectControlActualSpeed(IServiceDirectcontrol_ActualSpeedChanged);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error: " + ex.Message);
            }
        }


        private delegate void SetControlText(Control c, string text);
        private void DelegateImplementation(Control c, string text)
        {
            c.Text = text;
        }

        // Event handler for speed changes
        void IServiceDirectcontrol_ActualSpeedChanged(float newValue)
        {
            // Because of control changes have to be an the gui thread a invoke must be called to set the control text
            this.Invoke(new SetControlText(DelegateImplementation), new object[] { textBoxActualSpeed, newValue.ToString("N2") });
        }

        // Event handler for grade changes
        void IServiceDirectcontrol_ActualGradeChanged(float newValue)
        {
            // Because of control changes have to be an the gui thread a invoke must be called to set the control text
            this.Invoke(new SetControlText(DelegateImplementation), new object[] { textBoxActualGrade, newValue.ToString("N2") });
            
        }

        private void buttonDisconnect_Click(object sender, EventArgs e)
        {
            // Dispose the device
            if (mcu5device != null)
            {
                mcu5device.DisposeDevice();
                mcu5device = null;
            }
        }

        private void buttonSetSpeed_Click(object sender, EventArgs e)
        {
            if (IServiceDirectcontrol != null)
            {
                try
                {
                    // set a new drive speed
                    IServiceDirectcontrol.SetDriveSpeed(float.Parse(textBoxSpeed.Text), 3, 0);
                }
                catch (Exception er)
                {
                    MessageBox.Show("Error: " + er.Message);
                }
            }
        }

        private void buttonsetGrade_Click(object sender, EventArgs e)
        {
            if (IServiceDirectcontrol != null)
            {
                try
                {
                    // set a new elevator grade
                    IServiceDirectcontrol.SetElevatorGrade(float.Parse(textBoxGrade.Text));
                }
                catch (Exception er)
                {
                    MessageBox.Show("Error: " + er.Message);
                }
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Dispose the device
            if (mcu5device != null)
                mcu5device.DisposeDevice();

        }
    }
}